from django.contrib import admin
from .models import (
    Fach,
    Dozent,
    Testat,
    Frage,
)

class FachAdmin(admin.ModelAdmin):
    model = Fach
    list_display = ('name', 'admin_list_dozent')

admin.site.register(Fach, FachAdmin)

class DozentAdmin(admin.ModelAdmin):
    model = Dozent
    list_display = ('full_name', 'aktiv', 'fach')
    search_fields = ['vorname', 'nachname', 'fach__name']


admin.site.register(Dozent, DozentAdmin)


class TestatAdmin(admin.ModelAdmin):
    model = Testat
    list_display = (
        'name',
        'active',
        'admin_list_fach',
        'admin_list_studienabschnitt',
        'admin_list_studiengang',
    )
    filter_horizontal = ('fach', 'studiengang', 'studienabschnitt')
    search_fields = ['name']


admin.site.register(Testat, TestatAdmin)


class FrageAdmin(admin.ModelAdmin):
    model = Frage
    list_display = ('__str__', 'testat', 'pruefer', 'punkte', 'datum')
    filter_horizontal = ('abgestimmte_benutzer',)
    search_fields = [
        'text',
        'testat__name',
        'pruefer__nachname',
        'pruefer__fach__name',
    ]


admin.site.register(Frage, FrageAdmin)



from django.urls import path
from django.conf import settings
from django.conf.urls.static import static

from . import views

app_name = 'exoral'

urlpatterns = [
    path('', views.Home.as_view(), name='home'),
    path('downvote/<int:frage_id>', views.FrageDownvote.as_view(), name='downvote-frage'),
    path('dozent/<int:dozent_id>', views.DozentDetail.as_view(), name='dozent-detail'),
    path('dozent/<int:dozent_id>/edit', views.DozentEdit.as_view(), name='dozent-edit'),
    path('dozent/create', views.DozentCreate.as_view(), name='dozent-create'),
    path('dozenten', views.DozentList.as_view(), name='dozent-list'),
    path('fach/<int:fach_id>/edit', views.FachEdit.as_view(), name='fach-edit'),
    path('fach/create', views.FachCreate.as_view(), name='fach-create'),
    path('faecher', views.FachList.as_view(), name='fach-list'),
    path('frage/<int:frage_id>/delete', views.FrageDelete.as_view(), name='frage-delete'),
    path('frage/<int:frage_id>/edit', views.FrageEdit.as_view(), name='frage-edit'),
    path('frage/<int:testat_id>/<int:pruefer_id>/create', views.FrageCreate.as_view(), name='frage-create'),
    path('fragen/<int:testat_id>/<int:pruefer_id>', views.FrageList.as_view(), name='frage-list'),
    path('testat/<int:testat_id>', views.TestatDetail.as_view(), name='testat-detail'),
    path('testat/<int:testat_id>/edit', views.TestatEdit.as_view(), name='testat-edit'),
    path('testat/create', views.TestatCreate.as_view(), name='testat-create'),
    path('testate', views.TestatList.as_view(), name='testat-list'),
    path('upvote/<int:frage_id>', views.FrageUpvote.as_view(), name='upvote-frage'),
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



# Generated by Django 3.0.10 on 2020-11-07 18:31

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('fsmedhro_core', '0002_alter_field_user_on_fachschaftuser'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Dozent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('titel', models.CharField(blank=True, max_length=30)),
                ('vorname', models.CharField(blank=True, max_length=60)),
                ('nachname', models.CharField(max_length=100)),
                ('aktiv', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Dozent',
                'verbose_name_plural': 'Dozenten',
                'ordering': ('nachname',),
            },
        ),
        migrations.CreateModel(
            name='Fach',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Fach',
                'verbose_name_plural': 'Fächer',
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='Testat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('active', models.BooleanField(default=True)),
                ('fach', models.ManyToManyField(to='exoral.Fach')),
                ('studienabschnitt', models.ManyToManyField(to='fsmedhro_core.Studienabschnitt')),
                ('studiengang', models.ManyToManyField(to='fsmedhro_core.Studiengang')),
            ],
            options={
                'verbose_name': 'Testat',
                'verbose_name_plural': 'Testate',
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='Frage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('datum', models.DateField(default=datetime.date.today, verbose_name='Prüfungsdatum')),
                ('text', models.TextField()),
                ('antwort', models.TextField(blank=True)),
                ('punkte', models.PositiveIntegerField(default=1)),
                ('abgestimmte_benutzer', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
                ('pruefer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='exoral.Dozent', verbose_name='PrüferIn')),
                ('testat', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='exoral.Testat', verbose_name='Testat/Prüfung')),
            ],
            options={
                'verbose_name': 'Frage',
                'verbose_name_plural': 'Fragen',
            },
        ),
        migrations.AddField(
            model_name='dozent',
            name='fach',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='exoral.Fach'),
        ),
    ]



from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import render, get_object_or_404, redirect
from django.urls import reverse
from django.views import View
from django.views.generic import DetailView, ListView
from django.views.generic.edit import UpdateView, DeleteView, CreateView
from django.contrib import messages

from .models import (
    Dozent,
    Fach,
    Frage,
    Testat,
)

from fsmedhro_core.models import (
    FachschaftUser,
    Studienabschnitt,
    Studiengang,
)


class Home(LoginRequiredMixin, View):
    def get(self, request):
        user = request.user

        name = user.username
        if user.first_name:
            name = user.first_name

        context = {
            'name': name,
        }

        try:
            fs_user = FachschaftUser.objects.get(user=request.user)
            studiengang = fs_user.studiengang
            studienabschnitt = fs_user.studienabschnitt
        except (FachschaftUser.DoesNotExist, Studiengang.DoesNotExist,
                Studienabschnitt.DoesNotExist):
            return render(request, 'exoral/profil_unvollstaendig.html', context)

        testate = Testat.objects.filter(
            studiengang=studiengang,
            studienabschnitt=studienabschnitt,
        )

        letzte_fragen = Frage.objects.prefetch_related(
            'pruefer',
            'testat'
        ).order_by('-id')[:20]

        context.update(
            {
                'studiengang': studiengang,
                'studienabschnitt': studienabschnitt,
                'testate': testate,
                'letzte_fragen': letzte_fragen,
            }
        )

        return render(request, 'exoral/home.html', context)


# Testat

class TestatCreate(LoginRequiredMixin, PermissionRequiredMixin, CreateView):
    permission_required = 'exoral.add_testat'

    model = Testat
    fields = ['name', 'active', 'fach', 'studiengang', 'studienabschnitt']
    template_name_suffix = '_create'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:testat-list')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['faecher'] = Fach.objects.all()
        context['studiengaenge'] = Studiengang.objects.all()
        context['studienabschnitte'] = Studienabschnitt.objects.all()
        return context


class TestatEdit(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'exoral.change_testat'

    queryset = Testat.objects.prefetch_related(
        'fach',
        'studiengang',
        'studienabschnitt',
    )
    fields = ['name', 'active', 'fach', 'studiengang', 'studienabschnitt']
    pk_url_kwarg = 'testat_id'
    template_name_suffix = '_edit'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:testat-detail', kwargs={'testat_id': self.object.id})

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['faecher'] = Fach.objects.all()
        context['studiengaenge'] = Studiengang.objects.all()
        context['studienabschnitte'] = Studienabschnitt.objects.all()
        return context


class TestatDetail(LoginRequiredMixin, DetailView):
    model = Testat
    pk_url_kwarg = 'testat_id'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        faecher = dict()  # Dict[Fach → Dict[Dozent → Int]]
        testat = self.object
        fragen = list(
            Frage.objects.prefetch_related(
                'testat',
                'pruefer',
            ).filter(testat=testat)
        )

        for fach in testat.fach.prefetch_related('dozent_set'):
            faecher[fach] = dict()  # Dict[Dozent → int]
            for dozent in fach.dozent_set.all():
                faecher[fach][dozent] = len(
                    [frage for frage in fragen if frage.pruefer==dozent]
                )

        context['faecher'] = faecher
        return context


class TestatList(LoginRequiredMixin, ListView):
    queryset = Testat.objects.prefetch_related(
        'fach',
        'studiengang',
        'studienabschnitt',
        'frage_set',
    )
    context_object_name = 'testate'


# Frage

class FrageCreate(LoginRequiredMixin, View):
    model = Frage
    fields = ['datum', 'text', 'antwort']
    template_name = 'exoral/frage_form.html'

    def dispatch(self, request, *args, **kwargs):
        self.testat = get_object_or_404(Testat, pk=self.kwargs['testat_id'])
        self.pruefer = get_object_or_404(Dozent, pk=self.kwargs['pruefer_id'])
        if self.pruefer.fach not in self.testat.fach.all():
            error = (
                'Prüfer:in und Testat passen nicht zusammen. '
                'Das Fach des:r Prüfers:in muss in den Fächern des Testats '
                'enthalten sein.'
            )
            return render(request, 'exoral/error.html', {'error': error})
        else:
            return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        context = {
            'testat': self.testat,
            'pruefer': self.pruefer,
            'vorhandene_fragen': Frage.objects.filter(
                pruefer=self.pruefer,
                testat=self.testat,
            ),
        }
        return render(request, self.template_name, context)

    def post(self, request, *args, **kwargs):
        datum = request.POST.get('datum')
        text = request.POST.get('text').strip()
        antwort = request.POST.get('antwort')

        frage = Frage(
            datum=datum,
            text=text,
            antwort=antwort,
            pruefer=self.pruefer,
            testat=self.testat,
        )
        frage.save()

        frage.abgestimmte_benutzer.add(request.user)

        return redirect(
            'exoral:frage-list',
            testat_id=self.testat.pk,
            pruefer_id=self.pruefer.pk,
        )


class FrageEdit(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'exoral.change_frage'

    model = Frage
    fields = ['datum', 'text', 'antwort', 'pruefer', 'testat']
    pk_url_kwarg = 'frage_id'
    template_name_suffix = '_edit'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:frage-edit', kwargs={'frage_id': self.object.id})

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['dozenten'] = Dozent.objects.all()
        context['testate'] = Testat.objects.all()
        return context


class FrageList(LoginRequiredMixin, ListView):
    model = Frage
    context_object_name = 'fragen'

    def dispatch(self, request, *args, **kwargs):
        self.testat = get_object_or_404(Testat, pk=self.kwargs['testat_id'])
        self.pruefer = get_object_or_404(Dozent, pk=self.kwargs['pruefer_id'])
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        fragen = Frage.objects.filter(
            testat=self.testat,
            pruefer=self.pruefer,
        ).prefetch_related(
            'abgestimmte_benutzer'
        ).order_by(
            '-punkte', '-datum'
        )
        user = self.request.user

        for frage in fragen:
            frage.abgestimmt = user in frage.abgestimmte_benutzer.all()

        return fragen

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({'testat': self.testat, 'pruefer': self.pruefer})
        return context


class FrageDelete(LoginRequiredMixin, PermissionRequiredMixin, DeleteView):
    permission_required = 'exoral.delete_frage'

    model = Frage
    pk_url_kwarg = 'frage_id'

    def get_success_url(self):
        testat = self.object.testat
        pruefer = self.object.pruefer
        messages.add_message(self.request, messages.SUCCESS, 'Gelöscht!')
        return reverse(
            'exoral:frage-list',
            kwargs={'testat_id': testat.id, 'pruefer_id': pruefer.id}
        )


class FrageUpvote(LoginRequiredMixin, View):
    def get(self, request, frage_id):
        frage = get_object_or_404(Frage, pk=frage_id)
        frage.upvote(request.user)
        return redirect(
            'exoral:frage-list',
            testat_id=frage.testat.id,
            pruefer_id=frage.pruefer.id,
        )


class FrageDownvote(LoginRequiredMixin, View):
    def get(self, request, frage_id):
        frage = get_object_or_404(Frage, pk=frage_id)
        frage.downvote(request.user)
        return redirect(
            'exoral:frage-list',
            testat_id=frage.testat.id,
            pruefer_id=frage.pruefer.id,
        )


# Dozent

class DozentCreate(LoginRequiredMixin, PermissionRequiredMixin, CreateView):
    permission_required = 'exoral.add_dozent'
    model = Dozent
    fields = ['titel', 'vorname', 'nachname', 'aktiv', 'fach']
    template_name_suffix = '_create'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:dozent-list')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['faecher'] = Fach.objects.all()
        return context


class DozentEdit(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'exoral.change_dozent'

    model = Dozent
    fields = '__all__'
    pk_url_kwarg = 'dozent_id'
    template_name_suffix = '_edit'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:dozent-detail', kwargs={'dozent_id': self.object.id})

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['faecher'] = Fach.objects.all()
        return context


class DozentDetail(LoginRequiredMixin, DetailView):
    model = Dozent
    pk_url_kwarg = 'dozent_id'


class DozentList(LoginRequiredMixin, ListView):
    queryset = Dozent.objects.prefetch_related('fach')


# Fach

class FachCreate(LoginRequiredMixin, PermissionRequiredMixin, CreateView):
    permission_required = 'exoral.add_fach'
    model = Fach
    fields = ['name']
    template_name_suffix = '_create'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:fach-list')


class FachEdit(LoginRequiredMixin, PermissionRequiredMixin, UpdateView):
    permission_required = 'exoral.change_fach'

    model = Fach
    fields = ['name']
    pk_url_kwarg = 'fach_id'
    template_name_suffix = '_edit'

    def get_success_url(self):
        messages.add_message(self.request, messages.SUCCESS, 'Gespeichert!')
        return reverse('exoral:fach-list')


class FachList(LoginRequiredMixin, ListView):
    queryset = Fach.objects.prefetch_related('dozent_set', 'testat_set')



from django.db import models
from django.conf import settings
from datetime import date

from fsmedhro_core.models import Studienabschnitt, Studiengang


class Fach(models.Model):
    """
    Ein Fach besteht nur aus dem Namen des Fachs, z.B. Anatomie, Biochemie,
    Physiologie.
    """

    name = models.CharField(max_length=100)

    class Meta:
        verbose_name = "Fach"
        verbose_name_plural = "Fächer"
        ordering = ("name",)

    def __str__(self):
        return self.name

    def admin_list_dozent(self):
        return ', '.join(doz.nachname for doz in self.dozent_set.all())
    admin_list_dozent.short_description = 'Dozenten'


class Dozent(models.Model):
    """
    Nachname, Fach und aktiv sind zwingend erforderlich.
    Titel und Vorname sind optional.

    Ein Dozent ist immer genau einem Fach zugeordnet.
    """

    titel = models.CharField(max_length=30, blank=True)
    vorname = models.CharField(max_length=60, blank=True)
    nachname = models.CharField(max_length=100)
    aktiv = models.BooleanField(default=True)

    fach = models.ForeignKey(Fach, on_delete=models.CASCADE, blank=True)

    class Meta:
        verbose_name = "Dozent"
        verbose_name_plural = "Dozenten"
        ordering = ("nachname",)

    def name(self):
        """
        [Vorname] [Nachname]
        """
        if self.vorname:
            return f"{self.vorname} {self.nachname}"
        else:
            return self.nachname

    def full_name(self):
        """
        [Titel] [Vorname] [Nachname]
        """
        r = self.nachname

        if self.vorname:
            r = f"{self.vorname} {r}"
        if self.titel:
            r = f"{self.titel} {r}"

        return r

    full_name.short_description = "Name"

    def __str__(self):
        return self.full_name()


class Testat(models.Model):
    """
    Ein Testat hat einen Namen und ist aktiv oder nicht.

    Ein Testat kann verschiedenen Studiengängen zugeordnet werden (z.B. Human-
    und Zahnmedizin).
    Ein Testat kann verschiedenen Studienabschnitten zugeordnet werden (z.B. BP
    Chirurgie im 9. und 10. Semester).
    Ein Testat kann verschiedenen Fächern zugeordnet werden (z.B. das Physikum
    den Fächern Anatomie, Biochemie und Physiologie).
    """

    name = models.CharField(max_length=100)
    active = models.BooleanField(default=True)

    fach = models.ManyToManyField(Fach)
    studiengang = models.ManyToManyField(Studiengang)
    studienabschnitt = models.ManyToManyField(Studienabschnitt)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Testat"
        verbose_name_plural = "Testate"
        ordering = ("name",)

    def admin_list_fach(self):
        return ', '.join(fach.name for fach in self.fach.all())
    admin_list_fach.short_description = 'Fächer'

    def admin_list_studiengang(self):
        return ', '.join(studgang.bezeichnung for studgang in self.studiengang.all())
    admin_list_studiengang.short_description = 'Studiengänge'

    def admin_list_studienabschnitt(self):
        return ', '.join(studab.bezeichnung for studab in self.studienabschnitt.all())
    admin_list_studienabschnitt.short_description = 'Studienabschnitte'


class Frage(models.Model):
    datum = models.DateField(default=date.today, verbose_name="Prüfungsdatum")
    text = models.TextField()
    antwort = models.TextField(blank=True)
    punkte = models.PositiveIntegerField(default=1)

    pruefer = models.ForeignKey(
        Dozent,
        on_delete=models.CASCADE,
        verbose_name="PrüferIn",
    )
    testat = models.ForeignKey(
        Testat,
        on_delete=models.CASCADE,
        verbose_name="Testat/Prüfung",
    )
    abgestimmte_benutzer = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        blank=True,
    )

    def __str__(self):
        t = ' '.join(self.text.split()[:10])
        if len(self.text) > len(t):
            t += ' […]'
        return t

    class Meta:
        verbose_name = "Frage"
        verbose_name_plural = "Fragen"

    def upvote(self, user):
        if user not in self.abgestimmte_benutzer.all():
            self.abgestimmte_benutzer.add(user)
            self.punkte = models.F("punkte") + 1
            self.save()

    def downvote(self, user):
        if user in self.abgestimmte_benutzer.all():
            self.abgestimmte_benutzer.remove(user)
            self.punkte = models.F("punkte") - 1
            self.save()



# Generated by Django 3.0.10 on 2020-11-13 19:52

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('exoral', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='frage',
            name='abgestimmte_benutzer',
            field=models.ManyToManyField(blank=True, to=settings.AUTH_USER_MODEL),
        ),
    ]
